@@ -36,20 +36,28 @@
 #include "vinagre-bookmarks.h"
 #include "vinagre-fav.h"
 #include "vinagre-window-private.h"
-#include "vinagre-prefs-manager.h"
+#include "vinagre-prefs.h"
 
 void
 vinagre_cmd_direct_connect (VinagreConnection *conn,
 			    VinagreWindow     *window)
 {
-  GtkWidget *tab;
+  VinagreTab *tab;
 
   g_return_if_fail (VINAGRE_IS_WINDOW (window));
 
-  tab = vinagre_tab_new (conn, window);
-  vinagre_notebook_add_tab (VINAGRE_NOTEBOOK (window->priv->notebook),
-			    VINAGRE_TAB (tab),
-			    -1);
+  tab = vinagre_window_conn_exists (window, conn);
+  if (tab)
+    {
+      vinagre_window_set_active_tab (window, tab);
+    }
+  else
+    {
+      tab = VINAGRE_TAB (vinagre_tab_new (conn, window));
+      vinagre_notebook_add_tab (VINAGRE_NOTEBOOK (window->priv->notebook),
+				VINAGRE_TAB (tab),
+				-1);
+    }
 }
 
 /* Machine Menu */
@@ -57,7 +65,7 @@ void
 vinagre_cmd_machine_connect (GtkAction     *action,
 			     VinagreWindow *window)
 {
-  GtkWidget *tab;
+  VinagreTab *tab;
   VinagreConnection *conn;
 
   g_return_if_fail (VINAGRE_IS_WINDOW (window));
@@ -66,10 +74,18 @@ vinagre_cmd_machine_connect (GtkAction     *action,
   if (!conn)
     return;
 
-  tab = vinagre_tab_new (conn, window);
-  vinagre_notebook_add_tab (VINAGRE_NOTEBOOK (window->priv->notebook),
-			    VINAGRE_TAB (tab),
-			    -1);
+  tab = vinagre_window_conn_exists (window, conn);
+  if (tab)
+    {
+      vinagre_window_set_active_tab (window, tab);
+    }
+  else
+    {
+      tab = VINAGRE_TAB (vinagre_tab_new (conn, window));
+      vinagre_notebook_add_tab (VINAGRE_NOTEBOOK (window->priv->notebook),
+				VINAGRE_TAB (tab),
+				-1);
+    }
 }
 
 void
@@ -154,6 +170,13 @@ vinagre_cmd_machine_take_screenshot (GtkAction     *action,
   vinagre_tab_take_screenshot (vinagre_window_get_active_tab (window));
 }
 
+void
+vinagre_cmd_machine_send_ctrlaltdel (GtkAction     *action,
+				     VinagreWindow *window)
+{
+  vinagre_tab_send_ctrlaltdel (vinagre_window_get_active_tab (window));
+}
+
 void
 vinagre_cmd_machine_close_all (GtkAction     *action,
 			       VinagreWindow *window)
@@ -161,6 +184,13 @@ vinagre_cmd_machine_close_all (GtkAction     *action,
   vinagre_window_close_all_tabs (window);
 }
 
+void
+vinagre_cmd_machine_quit (GtkAction     *action,
+			  VinagreWindow *window)
+{
+  gtk_widget_destroy (GTK_WIDGET (window));
+}
+
 /* View Menu */
 void
 vinagre_cmd_view_show_toolbar	(GtkAction     *action,
@@ -170,7 +200,9 @@ vinagre_cmd_view_show_toolbar	(GtkAction     *action,
 
   vinagre_utils_toggle_widget_visible (window->priv->toolbar);
 
-  vinagre_prefs_manager_set_toolbar_visible (GTK_WIDGET_VISIBLE (window->priv->toolbar));
+  g_object_set (vinagre_prefs_get_default (),
+		"toolbar-visible", GTK_WIDGET_VISIBLE (window->priv->toolbar),
+		NULL);
 }
 
 void
@@ -181,7 +213,9 @@ vinagre_cmd_view_show_statusbar	(GtkAction     *action,
 
   vinagre_utils_toggle_widget_visible (window->priv->statusbar);
 
-  vinagre_prefs_manager_set_statusbar_visible (GTK_WIDGET_VISIBLE (window->priv->statusbar));
+  g_object_set (vinagre_prefs_get_default (),
+		"statusbar-visible", GTK_WIDGET_VISIBLE (window->priv->statusbar),
+		NULL);
 }
 
 void
@@ -192,7 +226,9 @@ vinagre_cmd_view_show_fav_panel	(GtkAction     *action,
 
   vinagre_utils_toggle_widget_visible (window->priv->fav_panel);
 
-  vinagre_prefs_manager_set_side_pane_visible (GTK_WIDGET_VISIBLE (window->priv->fav_panel));
+  g_object_set (vinagre_prefs_get_default (),
+		"side-panel-visible", GTK_WIDGET_VISIBLE (window->priv->fav_panel),
+		NULL);
 }
 
 void
@@ -209,26 +245,18 @@ vinagre_cmd_view_scaling (GtkAction     *action,
 			  VinagreWindow *window)
 {
   gboolean active;
+  VinagreTab *tab;
 
   g_return_if_fail (VINAGRE_IS_WINDOW (window));
 
+  tab = vinagre_window_get_active_tab (window);
+  if (!tab)
+    return;
+
   active = gtk_toggle_action_get_active (GTK_TOGGLE_ACTION (action));
 
-  if (active &&
-      gdk_screen_is_composited (gtk_widget_get_screen (GTK_WIDGET (window))))
-    {
-      gtk_toggle_action_set_active (GTK_TOGGLE_ACTION (action), FALSE);
-      vinagre_utils_show_error (_("Scaling does not work properly on composited windows. Disable the visual effects and try again."),
-				GTK_WINDOW (window));
-      return;
-    }
-
-  if (!vinagre_tab_set_scaling (vinagre_window_get_active_tab (window), active))
-    {
-      gtk_toggle_action_set_active (GTK_TOGGLE_ACTION (action), FALSE);
-      vinagre_utils_show_error (_("Scaling is not supported on this installation.\n\nRead the README file (shipped with Vinagre) in order to know how to enable this feature."),
-				GTK_WINDOW (window));
-    }
+  if (!vinagre_tab_set_scaling (tab, active))
+    gtk_toggle_action_set_active (GTK_TOGGLE_ACTION (action), FALSE);
 }
 
 void
@@ -251,17 +279,25 @@ void
 vinagre_cmd_open_bookmark (VinagreWindow     *window,
 			   VinagreConnection *conn)
 {
-  GtkWidget *tab;
+  VinagreTab *tab;
   VinagreConnection *new_conn;
 
   g_return_if_fail (VINAGRE_IS_WINDOW (window));
+  g_return_if_fail (VINAGRE_IS_CONNECTION (conn));
 
-  new_conn = vinagre_connection_clone (conn);
-  tab = vinagre_tab_new (new_conn, window);
-  vinagre_notebook_add_tab (VINAGRE_NOTEBOOK (window->priv->notebook),
-			    VINAGRE_TAB (tab),
-			    -1);
-  gtk_widget_show (tab);
+  tab = vinagre_window_conn_exists (window, conn);
+  if (tab)
+    {
+      vinagre_window_set_active_tab (window, tab);
+    }
+  else
+    {
+      new_conn = vinagre_connection_clone (conn);
+      tab = VINAGRE_TAB (vinagre_tab_new (new_conn, window));
+      vinagre_notebook_add_tab (VINAGRE_NOTEBOOK (window->priv->notebook),
+				VINAGRE_TAB (tab),
+				-1);
+    }
 }
 
 void
